// src/controllers/authController.js
const bcrypt = require('bcrypt');
const jwt = require('jsonwebtoken');
const db = require('../db/db'); // Import database query function

const JWT_SECRET = process.env.JWT_SECRET;
const SALT_ROUNDS = 12; // Cost factor for bcrypt hashing

// Register a new user
exports.register = async (req, res, next) => {
    const { username, password, role } = req.body; // Role might be optional or restricted

    // Basic validation
    if (!username || !password) {
        return res.status(400).json({ message: 'Benutzername und Passwort sind erforderlich.' });
    }

    // TODO: Add more robust validation (e.g., password complexity, username format)
    const validatePassword = (password) => {
        const minLength = 8;
        const hasUpperCase = /[A-Z]/.test(password);
        const hasLowerCase = /[a-z]/.test(password);
        const hasNumbers = /\d/.test(password);
        const hasSpecialChars = /[!@#$%^&*(),.?":{}|<>]/.test(password);
        
        return password.length >= minLength && hasUpperCase && 
               hasLowerCase && hasNumbers && hasSpecialChars;
      };

    // validatePassword(password) ? null : res.status(400).json({ message: 'Passwort muss mindestens 8 Zeichen lang sein und Großbuchstaben, Kleinbuchstaben, Zahlen und Sonderzeichen enthalten.' });

    // Determine the role - default to 'spectator' if not provided or invalid
    // In a real scenario, only admins should be able to assign 'admin' or 'referee' roles.
    const validRoles = ['admin', 'referee', 'spectator'];
    const assignedRole = validRoles.includes(role) ? role : 'spectator';

    // Check if user already exists
    try {
        const existingUser = await db.query('SELECT * FROM users WHERE username = $1', [username]);
        if (existingUser.rows.length > 0) {
            return res.status(409).json({ message: 'Benutzername bereits vergeben.' }); // Conflict
        }

        // Hash the password
        const passwordHash = await bcrypt.hash(password, SALT_ROUNDS);

        // Insert the new user into the database
        const newUser = await db.query(
            'INSERT INTO users (username, password_hash, role) VALUES ($1, $2, $3) RETURNING user_id, username, role, created_at',
            [username, passwordHash, assignedRole]
        );

        res.status(201).json({
            message: 'Benutzer erfolgreich registriert.',
            user: newUser.rows[0],
        });
    } catch (error) {
        // Pass error to the central error handler
        next(error);
    }
};

// Login a user
exports.login = async (req, res, next) => {
    const { username, password } = req.body;

    if (!username || !password) {
        return res.status(400).json({ message: 'Benutzername und Passwort sind erforderlich.' });
    }

    try {
        // Find the user by username
        const result = await db.query('SELECT user_id, username, password_hash, role FROM users WHERE username = $1', [username]);
        const user = result.rows[0];

        if (!user) {
            return res.status(401).json({ message: 'Ungültige Anmeldedaten.' }); // Unauthorized
        }

        // Compare the provided password with the stored hash
        const isMatch = await bcrypt.compare(password, user.password_hash);

        if (!isMatch) {
            return res.status(401).json({ message: 'Ungültige Anmeldedaten.' }); // Unauthorized
        }

        // Passwords match, create JWT payload
        const payload = {
            userId: user.user_id,
            username: user.username,
            role: user.role,
        };

        // Sign the JWT
        const token = jwt.sign(
            payload,
            JWT_SECRET,
            { expiresIn: '1h' } // Token expires in 1 hour (adjust as needed)
            // Consider adding refresh tokens for better security/UX
        );

        // Send the token back to the client
        res.json({
            message: 'Login erfolgreich.',
            token: token,
            user: { // Send back some user info (excluding password hash)
                userId: user.user_id,
                username: user.username,
                role: user.role
            }
        });

    } catch (error) {
        next(error);
    }
};
