/**
 * Error controller to standardize error handling across the application
 * Works alongside errorMiddleware.js but provides more granular control
 */

// Predefined error responses
const ErrorTypes = {
    VALIDATION_ERROR: 'VALIDATION_ERROR',
    DATABASE_ERROR: 'DATABASE_ERROR',
    NOT_FOUND: 'NOT_FOUND',
    UNAUTHORIZED: 'UNAUTHORIZED',
    FORBIDDEN: 'FORBIDDEN',
    CONFLICT: 'CONFLICT',
    INTERNAL_ERROR: 'INTERNAL_ERROR'
};

/**
 * Create a standard error response
 * @param {string} type - Error type from ErrorTypes
 * @param {string} message - Error message
 * @param {object} details - Additional error details (optional)
 * @param {number} status - HTTP status code (derived from type if not provided)
 * @returns {Error} Custom error object with additional properties
 */
const createError = (type, message, details = null, status = null) => {
    const error = new Error(message);
    error.type = type;
    error.details = details;
    
    // Set appropriate status code based on error type
    switch (type) {
        case ErrorTypes.VALIDATION_ERROR:
            error.statusCode = 400;
            break;
        case ErrorTypes.UNAUTHORIZED:
            error.statusCode = 401;
            break;
        case ErrorTypes.FORBIDDEN:
            error.statusCode = 403;
            break;
        case ErrorTypes.NOT_FOUND:
            error.statusCode = 404;
            break;
        case ErrorTypes.CONFLICT:
            error.statusCode = 409;
            break;
        case ErrorTypes.DATABASE_ERROR:
            error.statusCode = 500;
            break;
        case ErrorTypes.INTERNAL_ERROR:
            error.statusCode = 500;
            break;
        default:
            error.statusCode = 500;
    }
    
    // Override with custom status if provided
    if (status) {
        error.statusCode = status;
    }
    
    return error;
};

// Specific error creation helpers
const validationError = (message, details = null) => 
    createError(ErrorTypes.VALIDATION_ERROR, message, details);

const notFoundError = (resource, id = null) => {
    const message = id 
        ? `${resource} mit ID ${id} nicht gefunden.` 
        : `${resource} nicht gefunden.`;
    return createError(ErrorTypes.NOT_FOUND, message);
};

const conflictError = (message, details = null) =>
    createError(ErrorTypes.CONFLICT, message, details);

const databaseError = (error) => {
    let message = 'Datenbankfehler.';
    let details = null;
    
    // Extract detail from PostgreSQL error if available
    if (error.code) {
        details = { pgCode: error.code, pgDetail: error.detail };
        
        // Customize messages for common database errors
        switch (error.code) {
            case '23505': // unique_violation
                message = 'Ein Eintrag mit diesen Daten existiert bereits.';
                return createError(ErrorTypes.CONFLICT, message, details, 409);
            case '23503': // foreign_key_violation
                message = 'Der verknüpfte Eintrag existiert nicht oder kann nicht gelöscht werden.';
                return createError(ErrorTypes.VALIDATION_ERROR, message, details, 400);
            case '22P02': // invalid_text_representation (often for invalid UUIDs)
                message = 'Ungültiges Datenformat.';
                return createError(ErrorTypes.VALIDATION_ERROR, message, details, 400);
        }
    }
    
    return createError(ErrorTypes.DATABASE_ERROR, message, details);
};

const unauthorizedError = (message = 'Nicht autorisiert.') =>
    createError(ErrorTypes.UNAUTHORIZED, message);

const forbiddenError = (message = 'Zugriff verweigert.') =>
    createError(ErrorTypes.FORBIDDEN, message);

const internalError = (error) => {
    console.error('Internal Server Error:', error);
    return createError(
        ErrorTypes.INTERNAL_ERROR, 
        'Ein interner Serverfehler ist aufgetreten.',
        process.env.NODE_ENV === 'development' ? { stack: error.stack } : null
    );
};

module.exports = {
    ErrorTypes,
    createError,
    validationError,
    notFoundError,
    conflictError,
    databaseError,
    unauthorizedError,
    forbiddenError,
    internalError
};