// src/controllers/matchController.js
const db = require('../db/db');

// Helper function to get player names for matches (no changes needed here)
const enrichMatchesWithPlayerNames = async (matches) => {
    if (!matches || matches.length === 0) {
        return [];
    }
    const playerIds = new Set();
    matches.forEach(m => {
        if (m.player1_id) playerIds.add(m.player1_id);
        if (m.player2_id) playerIds.add(m.player2_id);
        if (m.winner_id) playerIds.add(m.winner_id);
    });

    if (playerIds.size === 0) return matches;

    const playerResult = await db.query(
        'SELECT player_id, first_name, last_name FROM players WHERE player_id = ANY($1::uuid[])',
        [Array.from(playerIds)]
    );
    const playerMap = new Map(playerResult.rows.map(p => [p.player_id, `${p.first_name} ${p.last_name}`]));

    return matches.map(m => ({
        ...m,
        player1_name: m.player1_id ? playerMap.get(m.player1_id) || 'Unbekannt' : null,
        player2_name: m.player2_id ? playerMap.get(m.player2_id) || 'Unbekannt' : null,
        winner_name: m.winner_id ? playerMap.get(m.winner_id) || 'Unbekannt' : null,
    }));
};


// Get all matches, filtered by tournament_id (optional) and optionally status
// If no tournamentId, fetches only 'scheduled' or 'ongoing' matches (for referee view)
exports.getAllMatches = async (req, res, next) => {
    const { tournamentId, status, refereeId } = req.query; // Added refereeId possibility

    let query = 'SELECT m.* FROM matches m'; // Base query
    const params = [];
    const conditions = [];
    let paramIndex = 1;

    if (tournamentId) {
        conditions.push(`m.tournament_id = $${paramIndex++}`);
        params.push(tournamentId);
    } else {
        // If no tournament ID, default to fetching only active matches for referee view
        conditions.push(`(m.status = 'scheduled' OR m.status = 'ongoing')`);
    }

    if (status) {
        conditions.push(`m.status = $${paramIndex++}`);
        params.push(status);
    }

    // TODO: Implement refereeId filtering if needed
    // if (refereeId === 'me' && req.user) { // Check if filtering by logged-in referee
    //     conditions.push(`m.referee_id = $${paramIndex++}`);
    //     params.push(req.user.userId);
    // } else if (refereeId) {
    //      conditions.push(`m.referee_id = $${paramIndex++}`);
    //      params.push(refereeId);
    // }


    if (conditions.length > 0) {
        query += ' WHERE ' + conditions.join(' AND ');
    }

    query += ' ORDER BY m.round NULLS LAST, m.match_number_in_round NULLS LAST, m.scheduled_time NULLS LAST, m.created_at';

    try {
        const result = await db.query(query, params);
        const enrichedMatches = await enrichMatchesWithPlayerNames(result.rows);
        res.json(enrichedMatches);
    } catch (error) {
        next(error);
    }
};

// Get a single match by ID
exports.getMatchById = async (req, res, next) => {
    const { id } = req.params;
     if (!id || id.toLowerCase() === 'export' || id.toLowerCase() === 'generate-bracket') { // Avoid conflict with potential routes
         return next();
     }
    try {
        // Fetch match and associated sets
        const matchResult = await db.query('SELECT * FROM matches WHERE match_id = $1', [id]);
        if (matchResult.rows.length === 0) {
            return res.status(404).json({ message: 'Spiel nicht gefunden.' });
        }

        const setsResult = await db.query('SELECT set_number, player1_score, player2_score FROM sets WHERE match_id = $1 ORDER BY set_number', [id]);

        const match = matchResult.rows[0];
        match.sets = setsResult.rows; // Add sets to the match object

        // Enrich with player names
        const [enrichedMatch] = await enrichMatchesWithPlayerNames([match]);

        res.json(enrichedMatch);
    } catch (error) {
        next(error);
    }
};

// Create a new match manually (Admin only - protected in router)
exports.createMatch = async (req, res, next) => {
    const {
        tournament_id,
        round,
        match_number_in_round,
        player1_id,
        player2_id,
        scheduled_time,
        table_number,
        status // Optional, defaults to 'scheduled'
    } = req.body;

    if (!tournament_id) {
        return res.status(400).json({ message: 'tournament_id ist erforderlich.' });
    }
     if (player1_id && player1_id === player2_id) {
         return res.status(400).json({ message: 'Spieler 1 und Spieler 2 dürfen nicht identisch sein.' });
     }

    try {
        const result = await db.query(
            `INSERT INTO matches (tournament_id, round, match_number_in_round, player1_id, player2_id, scheduled_time, table_number, status)
             VALUES ($1, $2, $3, $4, $5, $6, $7, $8)
             RETURNING *`,
            [
                tournament_id,
                round || null,
                match_number_in_round || null,
                player1_id || null,
                player2_id || null,
                scheduled_time || null,
                table_number || null,
                status || 'scheduled'
            ]
        );
         const [enrichedMatch] = await enrichMatchesWithPlayerNames(result.rows);
        res.status(201).json(enrichedMatch);
    } catch (error) {
         // Handle foreign key violation if tournament_id or player_ids are invalid
        if (error.code === '23503') {
            return res.status(400).json({ message: `Ungültige Turnier- oder Spieler-ID angegeben. (${error.detail})` });
        }
        next(error);
    }
};

// Update match details (Admin only - protected in router)
exports.updateMatch = async (req, res, next) => {
    const { id } = req.params;
    const {
        round,
        match_number_in_round,
        player1_id,
        player2_id,
        scheduled_time,
        status,
        table_number,
        referee_id,
        winner_id // Allow manual override
    } = req.body;

    const fieldsToUpdate = [];
    const values = [];
    let queryIndex = 1;

    const addUpdate = (field, value) => {
        if (value !== undefined) {
             fieldsToUpdate.push(`${field} = $${queryIndex++}`);
             values.push(value === '' ? null : value);
        }
    };

    try {
        // Fetch current match to validate player IDs if they are being changed
        const currentMatchRes = await db.query('SELECT player1_id, player2_id FROM matches WHERE match_id = $1', [id]);
         if (currentMatchRes.rows.length === 0) {
             return res.status(404).json({ message: 'Spiel nicht gefunden.' });
         }
         const currentMatch = currentMatchRes.rows[0];
         const p1 = player1_id !== undefined ? (player1_id || null) : currentMatch.player1_id; // Handle setting to null
         const p2 = player2_id !== undefined ? (player2_id || null) : currentMatch.player2_id;
         if (p1 && p2 && p1 === p2) {
             return res.status(400).json({ message: 'Spieler 1 und Spieler 2 dürfen nicht identisch sein.' });
         }

        addUpdate('round', round);
        addUpdate('match_number_in_round', match_number_in_round);
        addUpdate('player1_id', player1_id); // Allow null
        addUpdate('player2_id', player2_id); // Allow null
        addUpdate('scheduled_time', scheduled_time);
        addUpdate('status', status);
        addUpdate('table_number', table_number);
        addUpdate('referee_id', referee_id); // Allow null
        addUpdate('winner_id', winner_id); // Allow null


        if (fieldsToUpdate.length === 0) {
            return res.status(400).json({ message: 'Keine Daten zum Aktualisieren angegeben.' });
        }

        values.push(id); // Add match_id for WHERE clause

        const updateQuery = `UPDATE matches SET ${fieldsToUpdate.join(', ')}, updated_at = CURRENT_TIMESTAMP WHERE match_id = $${queryIndex} RETURNING *`;

        const result = await db.query(updateQuery, values);

        if (result.rows.length === 0) {
            return res.status(404).json({ message: 'Spiel nicht gefunden.' });
        }
        const [enrichedMatch] = await enrichMatchesWithPlayerNames(result.rows);
        res.json(enrichedMatch);

    } catch (error) {
        // Handle foreign key violation if player_ids/referee_id are invalid
        if (error.code === '23503') {
            return res.status(400).json({ message: `Ungültige Spieler- oder Schiedsrichter-ID angegeben. (${error.detail})` });
        }
        next(error);
    }
};

// Delete a match (Admin only - protected in router)
exports.deleteMatch = async (req, res, next) => {
    const { id } = req.params;
    try {
        // ON DELETE CASCADE for 'sets' table should handle associated set scores
        const result = await db.query('DELETE FROM matches WHERE match_id = $1 RETURNING match_id', [id]);
        if (result.rowCount === 0) {
            return res.status(404).json({ message: 'Spiel nicht gefunden.' });
        }
        res.status(204).send(); // No Content
    } catch (error) {
        next(error);
    }
};

// Enter/Update scores for a match (Admin or Referee - protected in router)
exports.updateMatchScore = async (req, res, next) => {
    const { id: matchId } = req.params;
    const { sets } = req.body; // Expecting an array: [{ set_number: 1, player1_score: 11, player2_score: 9 }, ...]

    if (!Array.isArray(sets)) { // Allow empty array to clear scores? No, require scores to update.
        return res.status(400).json({ message: 'Ein Array von Satzergebnissen ist erforderlich.' });
    }

    // --- Validation ---
    for (const set of sets) {
        const setNum = parseInt(set.set_number);
        const p1Score = parseInt(set.player1_score);
        const p2Score = parseInt(set.player2_score);
        if (
            set.set_number == null || set.player1_score == null || set.player2_score == null ||
            isNaN(setNum) || isNaN(p1Score) || isNaN(p2Score) ||
            setNum <= 0 || p1Score < 0 || p2Score < 0
        ) {
            return res.status(400).json({ message: `Ungültige Satzdaten: Satz ${set.set_number} (${set.player1_score}-${set.player2_score}). Jeder Satz benötigt set_number (>0), player1_score (>=0), player2_score (>=0).` });
        }
    }

    const client = await db.pool.connect(); // Use a transaction

    try {
        await client.query('BEGIN'); // Start transaction

        // 1. Fetch match details (including player IDs and tournament game type)
        const matchRes = await client.query(`
            SELECT m.match_id, m.player1_id, m.player2_id, m.status, t.game_type
            FROM matches m
            JOIN tournaments t ON m.tournament_id = t.tournament_id
            WHERE m.match_id = $1
            FOR UPDATE -- Lock the row during transaction
        `, [matchId]);

        if (matchRes.rows.length === 0) {
            await client.query('ROLLBACK');
            return res.status(404).json({ message: 'Spiel nicht gefunden.' });
        }
        const match = matchRes.rows[0];

        if (!match.player1_id || !match.player2_id) {
             await client.query('ROLLBACK');
             return res.status(400).json({ message: 'Spiel kann nicht bewertet werden, da nicht beide Spieler zugewiesen sind.' });
        }
         // Optional: Prevent updating scores if match is already 'finished'?
         // if (match.status === 'finished') {
         //     await client.query('ROLLBACK');
         //     return res.status(400).json({ message: 'Ergebnisse für bereits beendete Spiele können nicht geändert werden.' });
         // }


        // 2. Insert or Update Set Scores provided in the request
        // We only update the sets passed in the request body.
        for (const set of sets) {
            await client.query(
                `INSERT INTO sets (match_id, set_number, player1_score, player2_score)
                 VALUES ($1, $2, $3, $4)
                 ON CONFLICT (match_id, set_number)
                 DO UPDATE SET player1_score = EXCLUDED.player1_score,
                               player2_score = EXCLUDED.player2_score`,
                [matchId, parseInt(set.set_number), parseInt(set.player1_score), parseInt(set.player2_score)]
            );
        }

        // 3. Determine Winner based on ALL sets in the DB for this match after update
        const allSetsRes = await client.query('SELECT set_number, player1_score, player2_score FROM sets WHERE match_id = $1 ORDER BY set_number', [matchId]);
        const allSets = allSetsRes.rows;

        let player1SetsWon = 0;
        let player2SetsWon = 0;
        const pointsToWinSet = match.game_type === '21_points' ? 21 : 11;

        for (const s of allSets) {
             const p1 = s.player1_score;
             const p2 = s.player2_score;
             // Check win condition: must reach score limit AND win by 2 points
             const p1WinsSet = p1 >= pointsToWinSet && p1 >= p2 + 2;
             const p2WinsSet = p2 >= pointsToWinSet && p2 >= p1 + 2;

             if (p1WinsSet) {
                 player1SetsWon++;
             } else if (p2WinsSet) {
                 player2SetsWon++;
             }
             // If neither condition met (e.g., 10-10, 11-10, 20-20, 21-20), the set is not counted as won yet.
        }

        // Determine match winner based on sets won
        // TODO: Make "sets needed to win match" configurable per tournament
        const setsNeededToWinMatch = 3; // Assuming best of 5 for now
        let winnerId = null;
        let finalStatus = 'ongoing'; // Default status unless match is finished

        if (player1SetsWon >= setsNeededToWinMatch) {
            winnerId = match.player1_id;
            finalStatus = 'finished';
        } else if (player2SetsWon >= setsNeededToWinMatch) {
            winnerId = match.player2_id;
            finalStatus = 'finished';
        }
        // Check if max sets played (e.g., 5 sets) - if so, match is finished even if score is weird?
        // Or rely only on reaching setsNeededToWin. Let's stick to setsNeededToWin.

        // 4. Update Match Status and Winner
        await client.query(
            'UPDATE matches SET winner_id = $1, status = $2, updated_at = CURRENT_TIMESTAMP WHERE match_id = $3',
            [winnerId, finalStatus, matchId]
        );

        await client.query('COMMIT'); // Commit transaction

        // Fetch the updated match data to return
        const updatedMatchResult = await db.query('SELECT * FROM matches WHERE match_id = $1', [matchId]);
        const updatedSetsResult = await db.query('SELECT set_number, player1_score, player2_score FROM sets WHERE match_id = $1 ORDER BY set_number', [matchId]);
        const finalMatchData = updatedMatchResult.rows[0];
        finalMatchData.sets = updatedSetsResult.rows;
        const [enrichedFinalMatch] = await enrichMatchesWithPlayerNames([finalMatchData]);


        res.json({ message: 'Ergebnis erfolgreich gespeichert.', match: enrichedFinalMatch });

    } catch (error) {
        await client.query('ROLLBACK'); // Rollback transaction on error
        next(error);
    } finally {
        client.release(); // Release client back to the pool
    }
};


// --- Placeholder functions for other features ---

exports.exportMatchesPdf = async (req, res, next) => {
    // TODO: Implement PDF export logic using pdfkit
    res.status(501).json({ message: 'Funktion PDF-Export noch nicht implementiert.' });
};

exports.generateBracket = async (req, res, next) => {
    // TODO: Implement bracket generation logic
    const { tournamentId } = req.body;
     if (!tournamentId) return res.status(400).json({ message: "tournamentId ist erforderlich." });
    res.status(501).json({ message: 'Funktion Turnierbaum-Generierung noch nicht implementiert.' });
};
