// src/controllers/playerController.js
const db = require('../db/db');

// Get all players - includes search and gender
exports.getAllPlayers = async (req, res, next) => {
    const { search } = req.query;
    // Select gender along with other fields
    let query = 'SELECT player_id, first_name, last_name, gender, club, qttr_points, age_class FROM players';
    const params = [];

    if (search) {
        query += ` WHERE (first_name || ' ' || last_name) ILIKE $1
                     OR (last_name || ' ' || first_name) ILIKE $1
                     OR club ILIKE $1
                     OR age_class ILIKE $1`;
        params.push(`%${search}%`);
    }

    query += ' ORDER BY last_name, first_name';

    try {
        const result = await db.query(query, params);
        res.json(result.rows);
    } catch (error) {
        next(error);
    }
};

// Get a single player by ID - includes gender
exports.getPlayerById = async (req, res, next) => {
    const { id } = req.params;
     if (!id || id.toLowerCase() === 'export' || id.toLowerCase() === 'import') { return next(); }
    try {
        // Select gender
        const result = await db.query('SELECT player_id, first_name, last_name, gender, club, qttr_points, age_class, created_at, updated_at FROM players WHERE player_id = $1', [id]);
        if (result.rows.length === 0) {
            return res.status(404).json({ message: 'Spieler nicht gefunden.' });
        }
        res.json(result.rows[0]);
    } catch (error) {
        next(error);
    }
};

// Create a new player (Admin only) - includes gender
exports.createPlayer = async (req, res, next) => {
    const { first_name, last_name, gender, club, qttr_points, age_class } = req.body; // Added gender

    // Basic validation
    if (!first_name || !last_name) {
        return res.status(400).json({ message: 'Vorname und Nachname sind erforderlich.' });
    }
    // Validate gender
    const validGenders = ['m', 'w', 'd'];
    if (!gender || !validGenders.includes(gender)) {
        return res.status(400).json({ message: 'Gültiges Geschlecht (m, w, d) ist erforderlich.' });
    }

    const qtt_numeric = qttr_points ? parseInt(qttr_points, 10) : null;
    if (qttr_points && (isNaN(qtt_numeric) || qtt_numeric < 0)) {
        return res.status(400).json({ message: 'QTTR-Punkte müssen eine positive Zahl oder 0 sein.' });
    }

    try {
        const result = await db.query(
            // Include gender in INSERT and RETURNING
            `INSERT INTO players (first_name, last_name, gender, club, qttr_points, age_class)
             VALUES ($1, $2, $3, $4, $5, $6)
             RETURNING player_id, first_name, last_name, gender, club, qttr_points, age_class`,
            [first_name, last_name, gender, club || null, qtt_numeric, age_class || null]
        );
        res.status(201).json(result.rows[0]);
    } catch (error) {
         // Handle potential enum constraint violation for gender
         if (error.code === '23514' && error.constraint?.includes('gender')) { // Check error code and constraint name
             return res.status(400).json({ message: `Ungültiger Wert für Geschlecht: ${gender}. Erlaubt sind 'm', 'w', 'd'.` });
         }
        next(error);
    }
};

// Update an existing player (Admin only) - includes gender
exports.updatePlayer = async (req, res, next) => {
    const { id } = req.params;
    const { first_name, last_name, gender, club, qttr_points, age_class } = req.body; // Added gender

    // Build query dynamically
    const fieldsToUpdate = [];
    const values = [];
    let queryIndex = 1;

    const addUpdate = (field, value) => {
        if (value !== undefined) {
             let processedValue = value;
             if (field === 'qttr_points') {
                 processedValue = value !== null && value !== '' ? parseInt(value, 10) : null; // Allow setting to null
                 if (value !== null && value !== '' && (isNaN(processedValue) || processedValue < 0)) {
                      throw new Error('QTTR-Punkte müssen eine positive Zahl oder 0 sein.');
                 }
             } else if (field === 'gender') {
                  if (value && !['m', 'w', 'd'].includes(value)) {
                      throw new Error(`Ungültiges Geschlecht: ${value}. Erlaubt sind 'm', 'w', 'd'.`);
                  }
                  // Allow setting gender to null if needed? DB schema allows null currently.
                  processedValue = value || null;
             }
              else {
                 processedValue = (value === '' && (field === 'club' || field === 'age_class')) ? null : value;
             }
             fieldsToUpdate.push(`${field} = $${queryIndex++}`);
             values.push(processedValue);
        }
    };

    try {
        if (first_name === '') return res.status(400).json({ message: 'Vorname darf nicht leer sein.' });
        if (last_name === '') return res.status(400).json({ message: 'Nachname darf nicht leer sein.' });

        addUpdate('first_name', first_name);
        addUpdate('last_name', last_name);
        addUpdate('gender', gender); // Add gender update
        addUpdate('club', club);
        addUpdate('qttr_points', qttr_points);
        addUpdate('age_class', age_class);

        if (fieldsToUpdate.length === 0) {
            return res.status(400).json({ message: 'Keine Daten zum Aktualisieren angegeben.' });
        }

        values.push(id);

        // Include gender in RETURNING
        const updateQuery = `UPDATE players SET ${fieldsToUpdate.join(', ')}, updated_at = CURRENT_TIMESTAMP WHERE player_id = $${queryIndex} RETURNING player_id, first_name, last_name, gender, club, qttr_points, age_class`;

        const result = await db.query(updateQuery, values);

        if (result.rows.length === 0) {
            return res.status(404).json({ message: 'Spieler nicht gefunden.' });
        }
        res.json(result.rows[0]);

    } catch (error) {
        if (error.message.includes('QTTR-Punkte') || error.message.includes('Geschlecht')) {
            return res.status(400).json({ message: error.message });
        }
         // Handle potential enum constraint violation for gender on update
         if (error.code === '23514' && error.constraint?.includes('gender')) {
             return res.status(400).json({ message: `Ungültiger Wert für Geschlecht. Erlaubt sind 'm', 'w', 'd'.` });
         }
        next(error);
    }
};

// Delete a player (Admin only) - no changes needed for gender
exports.deletePlayer = async (req, res, next) => {
    const { id } = req.params;
    try {
        const checkResult = await db.query('SELECT 1 FROM tournament_players WHERE player_id = $1 LIMIT 1', [id]);
        if (checkResult.rows.length > 0) {
             return res.status(409).json({ message: 'Spieler kann nicht gelöscht werden, da er einem oder mehreren Turnieren zugeordnet ist.' });
        }
        const result = await db.query('DELETE FROM players WHERE player_id = $1 RETURNING player_id', [id]);
        if (result.rowCount === 0) {
            return res.status(404).json({ message: 'Spieler nicht gefunden.' });
        }
        res.status(204).send();
    } catch (error) {
        if (error.code === '23503') {
             console.warn(`Delete player ${id} FK violation: ${error.detail}`);
             return res.status(409).json({ message: 'Spieler konnte nicht gelöscht werden (Referenzfehler).' });
        }
        next(error);
    }
};


// --- Placeholder functions for Import/Export --- (no changes needed for gender here yet)
exports.importPlayers = async (req, res, next) => { /* ... placeholder ... */ res.status(501).json({ message: 'Funktion Spieler-Import noch nicht implementiert.' }); };
exports.exportPlayers = async (req, res, next) => { /* ... placeholder ... */
     try {
         const players = await db.query('SELECT first_name, last_name, gender, club, qttr_points, age_class FROM players ORDER BY last_name'); // Added gender
         if (players.rows.length === 0) { return res.status(404).json({ message: "Keine Spieler zum Exportieren gefunden." }); }
         res.setHeader('Content-Type', 'application/json'); res.setHeader('Content-Disposition', 'attachment; filename="spieler_export.json"'); res.json(players.rows);
    } catch (error) { next(error); }
};
