// src/controllers/tournamentController.js
const db = require('../db/db');

// --- Bestehende Funktionen (getAllTournaments, getTournamentById, createTournament, updateTournament, deleteTournament) ---
// ... (Code von tt_tournament_tournament_controller_v3 bleibt hier unverändert) ...
// Get all tournaments
exports.getAllTournaments = async (req, res, next) => {
    try {
        const result = await db.query('SELECT * FROM tournaments ORDER BY date DESC NULLS LAST, created_at DESC');
        res.json(result.rows);
    } catch (error) { next(error); }
};
// Get a single tournament by ID
exports.getTournamentById = async (req, res, next) => {
    const { id } = req.params;
    // Updated check to avoid conflict with '/export' or '/:id/players'
    if (!id || ['export', 'import'].includes(id.toLowerCase()) || req.path.includes('/players')) {
         return next();
    }
    try {
        const result = await db.query('SELECT * FROM tournaments WHERE tournament_id = $1', [id]);
        if (result.rows.length === 0) { return res.status(404).json({ message: 'Turnier nicht gefunden.' }); }
        res.json(result.rows[0]);
    } catch (error) { next(error); }
};
// Create a new tournament (Admin only - protected in router)
exports.createTournament = async (req, res, next) => {
    const { name, date, location, description, logo_url, tournament_type, game_type, max_players, status } = req.body;
    if (!name || !tournament_type || !game_type) { return res.status(400).json({ message: 'Name, Turnier-Typ und Spiel-Typ sind erforderlich.' }); }
    const validTournamentTypes = ['knockout', 'group']; const validGameTypes = ['11_points', '21_points']; const validStatuses = ['planned', 'running', 'finished', 'cancelled'];
    if (!validTournamentTypes.includes(tournament_type)) { return res.status(400).json({ message: `Ungültiger Turnier-Typ: ${tournament_type}` }); }
    if (!validGameTypes.includes(game_type)) { return res.status(400).json({ message: `Ungültiger Spiel-Typ: ${game_type}` }); }
    const finalStatus = status && validStatuses.includes(status) ? status : 'planned';
    const finalMaxPlayers = max_players ? parseInt(max_players, 10) : null;
    if (max_players && (isNaN(finalMaxPlayers) || finalMaxPlayers < 0)) { return res.status(400).json({ message: 'Max. Spieler muss eine positive Zahl sein.' }); }
    const created_by = req.user.userId;
    try {
        const result = await db.query(
            `INSERT INTO tournaments (name, date, location, description, logo_url, tournament_type, game_type, max_players, status, created_by)
             VALUES ($1, $2, $3, $4, $5, $6, $7, $8, $9, $10) RETURNING *`,
            [name, date || null, location, description, logo_url, tournament_type, game_type, finalMaxPlayers, finalStatus, created_by]
        );
        res.status(201).json(result.rows[0]);
    } catch (error) { next(error); }
};
// Update an existing tournament (Admin only - protected in router)
exports.updateTournament = async (req, res, next) => {
    const { id } = req.params;
    const { name, date, location, description, logo_url, tournament_type, game_type, max_players, status } = req.body;
    const fieldsToUpdate = []; const values = []; let queryIndex = 1;
    const addUpdate = (field, value) => {
        if (value !== undefined) {
            let processedValue = value;
            if (field === 'tournament_type' && !['knockout', 'group'].includes(value)) { throw new Error(`Ungültiger Turnier-Typ: ${value}`); }
            if (field === 'game_type' && !['11_points', '21_points'].includes(value)) { throw new Error(`Ungültiger Spiel-Typ: ${value}`); }
            if (field === 'status' && !['planned', 'running', 'finished', 'cancelled'].includes(value)) { throw new Error(`Ungültiger Status: ${value}`); }
            if (field === 'max_players') { processedValue = value ? parseInt(value, 10) : null; if (value && (isNaN(processedValue) || processedValue < 0)) { throw new Error('Max. Spieler muss eine positive Zahl sein.'); } }
            fieldsToUpdate.push(`${field} = $${queryIndex++}`); values.push(processedValue === '' ? null : processedValue);
        }
    };
    try {
        addUpdate('name', name); addUpdate('date', date); addUpdate('location', location); addUpdate('description', description); addUpdate('logo_url', logo_url);
        addUpdate('tournament_type', tournament_type); addUpdate('game_type', game_type); addUpdate('max_players', max_players); addUpdate('status', status);
        if (fieldsToUpdate.length === 0) { return res.status(400).json({ message: 'Keine Daten zum Aktualisieren angegeben.' }); }
        values.push(id);
        const updateQuery = `UPDATE tournaments SET ${fieldsToUpdate.join(', ')}, updated_at = CURRENT_TIMESTAMP WHERE tournament_id = $${queryIndex} RETURNING *`;
        const result = await db.query(updateQuery, values);
        if (result.rows.length === 0) { return res.status(404).json({ message: 'Turnier nicht gefunden.' }); }
        res.json(result.rows[0]);
    } catch (error) {
         if (error.message.startsWith('Ungültiger') || error.message.includes('Spieler')) { return res.status(400).json({ message: error.message }); }
        next(error);
    }
};
// Delete a tournament (Admin only - protected in router)
exports.deleteTournament = async (req, res, next) => {
    const { id } = req.params;
    try {
        const result = await db.query('DELETE FROM tournaments WHERE tournament_id = $1 RETURNING tournament_id', [id]);
        if (result.rowCount === 0) { return res.status(404).json({ message: 'Turnier nicht gefunden.' }); }
        res.status(204).send();
    } catch (error) { next(error); }
};
// Get players participating in a specific tournament - includes gender
exports.getTournamentPlayers = async (req, res, next) => {
    const { id } = req.params; // Tournament ID
    try {
        const result = await db.query(
            `SELECT p.player_id, p.first_name, p.last_name, p.gender, p.club, p.qttr_points, p.age_class,
                    tp.seed, tp.group_name, tp.status AS participation_status
             FROM players p
             JOIN tournament_players tp ON p.player_id = tp.player_id
             WHERE tp.tournament_id = $1
             ORDER BY p.last_name, p.first_name`,
            [id]
        );
        res.json(result.rows);
    } catch (error) { next(error); }
};


// --- NEUE FUNKTIONEN ---

/**
 * Fügt einen existierenden Spieler zu einem Turnier hinzu.
 * Erwartet playerId und optional seed, group_name im Body.
 * tournamentId kommt aus den URL-Parametern.
 * Admin only - geschützt in der Route.
 */
exports.addPlayerToTournament = async (req, res, next) => {
    const { tournamentId } = req.params;
    const { playerId, seed, group_name } = req.body;

    if (!playerId) {
        return res.status(400).json({ message: 'playerId ist im Request Body erforderlich.' });
    }

    // Optional: Validate if tournament and player actually exist before inserting?
    // Could add checks here, but FK constraints will catch invalid IDs anyway.

    try {
        const result = await db.query(
            `INSERT INTO tournament_players (tournament_id, player_id, seed, group_name)
             VALUES ($1, $2, $3, $4)
             ON CONFLICT (tournament_id, player_id) DO NOTHING
             RETURNING tournament_player_id`, // Check if something was returned
            [tournamentId, playerId, seed || null, group_name || null]
        );

        if (result.rows.length > 0) {
            // Successfully inserted
            res.status(201).json({ message: 'Spieler erfolgreich zum Turnier hinzugefügt.', added: true, detail: result.rows[0] });
        } else {
            // Conflict occurred, player was already in the tournament
             res.status(200).json({ message: 'Spieler ist bereits in diesem Turnier registriert.', added: false });
        }
    } catch (error) {
         // Handle foreign key violation if tournamentId or playerId is invalid
        if (error.code === '23503') {
            return res.status(404).json({ message: `Turnier oder Spieler nicht gefunden. (${error.detail})` });
        }
        next(error);
    }
};

/**
 * Entfernt einen Spieler aus einem Turnier.
 * tournamentId und playerId kommen aus den URL-Parametern.
 * Admin only - geschützt in der Route.
 */
exports.removePlayerFromTournament = async (req, res, next) => {
    const { tournamentId, playerId } = req.params;

    if (!tournamentId || !playerId) {
        // Should be guaranteed by route structure, but check anyway
        return res.status(400).json({ message: 'tournamentId und playerId sind in der URL erforderlich.' });
    }

    try {
        // Check if player is currently in a match that is not finished? Optional constraint.

        const result = await db.query(
            'DELETE FROM tournament_players WHERE tournament_id = $1 AND player_id = $2',
            [tournamentId, playerId]
        );

        if (result.rowCount === 0) {
            // No row was deleted, meaning the player wasn't in this tournament
            return res.status(404).json({ message: 'Spieler wurde in diesem Turnier nicht gefunden.' });
        }

        // Successfully deleted
        res.status(204).send(); // No Content

    } catch (error) {
        // Handle potential errors (though FK errors are less likely on DELETE here)
        next(error);
    }
};


// --- Placeholder functions --- (no changes)
exports.importTournaments = async (req, res, next) => { /* ... placeholder ... */ res.status(501).json({ message: 'Funktion noch nicht implementiert (Import).' }); };
exports.exportTournaments = async (req, res, next) => { /* ... placeholder ... */ res.status(501).json({ message: 'Funktion noch nicht implementiert (Export).' }); };
exports.addLogo = async (req, res, next) => { /* ... placeholder ... */ res.status(501).json({ message: 'Funktion noch nicht implementiert (Logo Upload).' }); };

