// src/controllers/userController.js
const db = require('../db/db');
const bcrypt = require('bcrypt');

const SALT_ROUNDS = 10; // Must match authController

// Function to create the initial admin user if none exist
exports.createInitialAdminUser = async () => {
    try {
        const result = await db.query('SELECT COUNT(*) FROM users');
        const userCount = parseInt(result.rows[0].count, 10);

        if (userCount === 0) {
            console.log('No users found. Creating initial admin user...');
            const adminUsername = 'admin';
            const adminPassword = 'password'; // !!! CHANGE THIS IN PRODUCTION !!!
            const adminRole = 'admin';

            const passwordHash = await bcrypt.hash(adminPassword, SALT_ROUNDS);

            await db.query(
                'INSERT INTO users (username, password_hash, role) VALUES ($1, $2, $3)',
                [adminUsername, passwordHash, adminRole]
            );
            console.log(`Initial admin user created: username='${adminUsername}', password='${adminPassword}'.`);
            console.warn("!!! IMPORTANT: Change the default admin password immediately! !!!");
        } else {
            console.log('Users already exist in the database. Skipping initial admin creation.');
        }
    } catch (error) {
        console.error('Error creating initial admin user:', error);
        // Decide if the application should exit or continue if this fails
    }
};


// --- User Management API Functions (Example) ---

// Get all users (Admin only)
exports.getAllUsers = async (req, res, next) => {
    try {
        // Select relevant fields, exclude password_hash
        const result = await db.query('SELECT user_id, username, role, created_at FROM users ORDER BY username');
        res.json(result.rows);
    } catch (error) {
        next(error);
    }
};

// Get a single user by ID (Admin only)
exports.getUserById = async (req, res, next) => {
    const { id } = req.params;
    try {
        const result = await db.query('SELECT user_id, username, role, created_at FROM users WHERE user_id = $1', [id]);
        if (result.rows.length === 0) {
            return res.status(404).json({ message: 'Benutzer nicht gefunden.' });
        }
        res.json(result.rows[0]);
    } catch (error) {
        next(error);
    }
};

// Create a new user (Admin only) - Similar to register but called by an admin
exports.createUser = async (req, res, next) => {
     const { username, password, role } = req.body;

    if (!username || !password || !role) {
        return res.status(400).json({ message: 'Benutzername, Passwort und Rolle sind erforderlich.' });
    }
     const validRoles = ['admin', 'referee', 'spectator'];
     if (!validRoles.includes(role)) {
         return res.status(400).json({ message: 'Ungültige Rolle angegeben.' });
     }

    try {
        // Check if username exists
        const existingUser = await db.query('SELECT 1 FROM users WHERE username = $1', [username]);
        if (existingUser.rows.length > 0) {
            return res.status(409).json({ message: 'Benutzername bereits vergeben.' });
        }

        const passwordHash = await bcrypt.hash(password, SALT_ROUNDS);
        const newUser = await db.query(
            'INSERT INTO users (username, password_hash, role) VALUES ($1, $2, $3) RETURNING user_id, username, role, created_at',
            [username, passwordHash, role]
        );
        res.status(201).json(newUser.rows[0]);
    } catch (error) {
        next(error);
    }
};

// Update a user (Admin only) - e.g., change role or reset password
exports.updateUser = async (req, res, next) => {
    const { id } = req.params;
    const { username, role, password } = req.body; // Allow updating username, role, password

    // Basic validation
    if (!username && !role && !password) {
        return res.status(400).json({ message: 'Keine Daten zum Aktualisieren angegeben.' });
    }

    const fieldsToUpdate = [];
    const values = [];
    let queryIndex = 1;

    if (username) {
        fieldsToUpdate.push(`username = $${queryIndex++}`);
        values.push(username);
    }
    if (role) {
         const validRoles = ['admin', 'referee', 'spectator'];
         if (!validRoles.includes(role)) {
             return res.status(400).json({ message: 'Ungültige Rolle angegeben.' });
         }
        fieldsToUpdate.push(`role = $${queryIndex++}`);
        values.push(role);
    }
    if (password) {
        // Hash new password if provided
        try {
            const passwordHash = await bcrypt.hash(password, SALT_ROUNDS);
            fieldsToUpdate.push(`password_hash = $${queryIndex++}`);
            values.push(passwordHash);
        } catch (hashError) {
            return next(hashError); // Pass hashing error
        }
    }

    // Add the user ID for the WHERE clause
    values.push(id);

    if (fieldsToUpdate.length === 0) {
         return res.status(400).json({ message: 'Keine gültigen Felder zum Aktualisieren angegeben.' });
    }

    const updateQuery = `UPDATE users SET ${fieldsToUpdate.join(', ')}, updated_at = CURRENT_TIMESTAMP WHERE user_id = $${queryIndex} RETURNING user_id, username, role, updated_at`;

    try {
        const result = await db.query(updateQuery, values);
        if (result.rows.length === 0) {
            return res.status(404).json({ message: 'Benutzer nicht gefunden.' });
        }
        res.json(result.rows[0]);
    } catch (error) {
        // Handle potential unique constraint violation for username
        if (error.code === '23505' && error.constraint === 'users_username_key') {
             return res.status(409).json({ message: 'Benutzername bereits vergeben.' });
        }
        next(error);
    }
};


// Delete a user (Admin only)
exports.deleteUser = async (req, res, next) => {
    const { id } = req.params;

    // Optional: Prevent admin from deleting themselves?
    // if (req.user.userId === id) {
    //     return res.status(403).json({ message: "Admins können sich nicht selbst löschen." });
    // }

    try {
        const result = await db.query('DELETE FROM users WHERE user_id = $1 RETURNING user_id', [id]);
        if (result.rowCount === 0) {
            return res.status(404).json({ message: 'Benutzer nicht gefunden.' });
        }
        res.status(204).send(); // No Content
    } catch (error) {
        next(error);
    }
};
