// src/db/db.js
const { Pool } = require('pg');
const fs = require('fs');
const path = require('path');

// Create a new PostgreSQL connection pool
// It reads connection details from environment variables:
// PGUSER, PGHOST, PGDATABASE, PGPASSWORD, PGPORT
// Or from DATABASE_URL environment variable.
const pool = new Pool({
    connectionString: process.env.DATABASE_URL,
    // Optional: Add SSL configuration if required by your PostgreSQL server
    // ssl: {
    //   rejectUnauthorized: false // Necessary for self-signed certificates or some cloud providers
    // }
});

// Function to execute the initialization script
const initializeDatabase = async () => {
    const client = await pool.connect();
    try {
        console.log('Checking database schema...');
        // Read the init.sql file
        const initSqlPath = path.join(__dirname, 'init.sql');
        const initSql = fs.readFileSync(initSqlPath, 'utf8');

        // Execute the SQL script
        // This script uses "CREATE TABLE IF NOT EXISTS" and "CREATE TYPE IF NOT EXISTS"
        // so it's safe to run multiple times.
        await client.query(initSql);
        console.log('Database schema check/initialization complete.');

    } catch (err) {
        console.error('Error during database initialization:', err.stack);
        throw err; // Re-throw error to be caught by server start logic
    } finally {
        client.release(); // Release the client back to the pool
    }
};

// Export the pool and the initialization function
module.exports = {
    query: (text, params) => pool.query(text, params), // Method to run queries
    pool, // Export the pool itself if needed elsewhere
    initializeDatabase,
};
