-- Database Initialization Script for Tischtennis-Turnierverwaltung (v2)

-- Enable UUID generation
CREATE EXTENSION IF NOT EXISTS "uuid-ossp";

-- User Roles Enum
DO $$ BEGIN
    CREATE TYPE user_role AS ENUM ('admin', 'referee', 'spectator');
EXCEPTION WHEN duplicate_object THEN null; END $$;

-- Gender Enum (NEW)
DO $$ BEGIN
    CREATE TYPE player_gender AS ENUM ('m', 'w', 'd'); -- m: männlich, w: weiblich, d: divers
EXCEPTION WHEN duplicate_object THEN null; END $$;


-- Users Table
CREATE TABLE IF NOT EXISTS users (
    user_id UUID PRIMARY KEY DEFAULT uuid_generate_v4(),
    username VARCHAR(50) UNIQUE NOT NULL,
    password_hash VARCHAR(255) NOT NULL,
    role user_role NOT NULL DEFAULT 'spectator',
    created_at TIMESTAMPTZ DEFAULT CURRENT_TIMESTAMP,
    updated_at TIMESTAMPTZ DEFAULT CURRENT_TIMESTAMP
);

-- Function to update 'updated_at' timestamp
CREATE OR REPLACE FUNCTION trigger_set_timestamp()
RETURNS TRIGGER AS $$ BEGIN NEW.updated_at = NOW(); RETURN NEW; END; $$ LANGUAGE plpgsql;

-- Trigger for users table
DO $$ BEGIN CREATE TRIGGER set_timestamp_users BEFORE UPDATE ON users FOR EACH ROW EXECUTE FUNCTION trigger_set_timestamp();
EXCEPTION WHEN duplicate_object THEN null; END $$;


-- Tournament Type Enum
DO $$ BEGIN CREATE TYPE tournament_type AS ENUM ('knockout', 'group');
EXCEPTION WHEN duplicate_object THEN null; END $$;

-- Game Type Enum
DO $$ BEGIN CREATE TYPE game_type AS ENUM ('11_points', '21_points');
EXCEPTION WHEN duplicate_object THEN null; END $$;


-- Tournaments Table
CREATE TABLE IF NOT EXISTS tournaments (
    tournament_id UUID PRIMARY KEY DEFAULT uuid_generate_v4(),
    name VARCHAR(255) NOT NULL,
    date DATE,
    location VARCHAR(255),
    description TEXT,
    logo_url VARCHAR(255),
    tournament_type tournament_type NOT NULL DEFAULT 'knockout',
    game_type game_type NOT NULL DEFAULT '11_points',
    max_players INTEGER,
    status VARCHAR(50) DEFAULT 'planned', -- planned, running, finished, cancelled
    created_at TIMESTAMPTZ DEFAULT CURRENT_TIMESTAMP,
    updated_at TIMESTAMPTZ DEFAULT CURRENT_TIMESTAMP,
    created_by UUID REFERENCES users(user_id) ON DELETE SET NULL
);

-- Trigger for tournaments table
DO $$ BEGIN CREATE TRIGGER set_timestamp_tournaments BEFORE UPDATE ON tournaments FOR EACH ROW EXECUTE FUNCTION trigger_set_timestamp();
EXCEPTION WHEN duplicate_object THEN null; END $$;


-- Players Table (Added gender)
CREATE TABLE IF NOT EXISTS players (
    player_id UUID PRIMARY KEY DEFAULT uuid_generate_v4(),
    first_name VARCHAR(100) NOT NULL,
    last_name VARCHAR(100) NOT NULL,
    gender player_gender, -- NEW: Added gender column
    club VARCHAR(150),
    qttr_points INTEGER,
    age_class VARCHAR(50),
    created_at TIMESTAMPTZ DEFAULT CURRENT_TIMESTAMP,
    updated_at TIMESTAMPTZ DEFAULT CURRENT_TIMESTAMP
);

-- Trigger for players table
DO $$ BEGIN CREATE TRIGGER set_timestamp_players BEFORE UPDATE ON players FOR EACH ROW EXECUTE FUNCTION trigger_set_timestamp();
EXCEPTION WHEN duplicate_object THEN null; END $$;


-- Tournament Players Table (Many-to-Many relationship)
CREATE TABLE IF NOT EXISTS tournament_players (
    tournament_player_id UUID PRIMARY KEY DEFAULT uuid_generate_v4(),
    tournament_id UUID NOT NULL REFERENCES tournaments(tournament_id) ON DELETE CASCADE,
    player_id UUID NOT NULL REFERENCES players(player_id) ON DELETE CASCADE,
    seed INTEGER,
    group_name VARCHAR(50),
    status VARCHAR(50) DEFAULT 'registered', -- registered, active, eliminated
    UNIQUE (tournament_id, player_id),
    created_at TIMESTAMPTZ DEFAULT CURRENT_TIMESTAMP
);


-- Matches Table
CREATE TABLE IF NOT EXISTS matches (
    match_id UUID PRIMARY KEY DEFAULT uuid_generate_v4(),
    tournament_id UUID NOT NULL REFERENCES tournaments(tournament_id) ON DELETE CASCADE,
    round INTEGER,
    match_number_in_round INTEGER,
    player1_id UUID REFERENCES players(player_id) ON DELETE SET NULL,
    player2_id UUID REFERENCES players(player_id) ON DELETE SET NULL,
    winner_id UUID REFERENCES players(player_id) ON DELETE SET NULL,
    scheduled_time TIMESTAMPTZ,
    status VARCHAR(50) DEFAULT 'scheduled', -- scheduled, ongoing, finished, postponed
    table_number VARCHAR(20),
    referee_id UUID REFERENCES users(user_id) ON DELETE SET NULL,
    created_at TIMESTAMPTZ DEFAULT CURRENT_TIMESTAMP,
    updated_at TIMESTAMPTZ DEFAULT CURRENT_TIMESTAMP
);

-- Trigger for matches table
DO $$ BEGIN CREATE TRIGGER set_timestamp_matches BEFORE UPDATE ON matches FOR EACH ROW EXECUTE FUNCTION trigger_set_timestamp();
EXCEPTION WHEN duplicate_object THEN null; END $$;


-- Sets Table
CREATE TABLE IF NOT EXISTS sets (
    set_id UUID PRIMARY KEY DEFAULT uuid_generate_v4(),
    match_id UUID NOT NULL REFERENCES matches(match_id) ON DELETE CASCADE,
    set_number INTEGER NOT NULL CHECK (set_number > 0),
    player1_score INTEGER NOT NULL CHECK (player1_score >= 0),
    player2_score INTEGER NOT NULL CHECK (player2_score >= 0),
    created_at TIMESTAMPTZ DEFAULT CURRENT_TIMESTAMP,
    UNIQUE (match_id, set_number)
);


-- Notifications Table
CREATE TABLE IF NOT EXISTS notifications (
    notification_id UUID PRIMARY KEY DEFAULT uuid_generate_v4(),
    tournament_id UUID REFERENCES tournaments(tournament_id) ON DELETE CASCADE,
    message TEXT NOT NULL,
    type VARCHAR(50) DEFAULT 'info',
    is_sticky BOOLEAN DEFAULT FALSE,
    target_audience VARCHAR(50) DEFAULT 'all',
    created_at TIMESTAMPTZ DEFAULT CURRENT_TIMESTAMP,
    expires_at TIMESTAMPTZ
);


-- Add Indexes for performance
CREATE INDEX IF NOT EXISTS idx_users_username ON users(username);
CREATE INDEX IF NOT EXISTS idx_tournaments_date ON tournaments(date);
CREATE INDEX IF NOT EXISTS idx_tournaments_status ON tournaments(status); -- Index status for finding active tournaments
CREATE INDEX IF NOT EXISTS idx_players_last_name ON players(last_name);
CREATE INDEX IF NOT EXISTS idx_tournament_players_tournament ON tournament_players(tournament_id);
CREATE INDEX IF NOT EXISTS idx_tournament_players_player ON tournament_players(player_id);
CREATE INDEX IF NOT EXISTS idx_matches_tournament ON matches(tournament_id);
CREATE INDEX IF NOT EXISTS idx_matches_status ON matches(status);
CREATE INDEX IF NOT EXISTS idx_sets_match ON sets(match_id);


-- Optional: Table for storing settings
CREATE TABLE IF NOT EXISTS application_settings (
    setting_key VARCHAR(100) PRIMARY KEY,
    setting_value TEXT,
    description TEXT,
    updated_at TIMESTAMPTZ DEFAULT CURRENT_TIMESTAMP
);

-- Trigger for application_settings table
DO $$ BEGIN CREATE TRIGGER set_timestamp_application_settings BEFORE UPDATE ON application_settings FOR EACH ROW EXECUTE FUNCTION trigger_set_timestamp();
EXCEPTION WHEN duplicate_object THEN null; END $$;

