// src/middleware/authMiddleware.js
const jwt = require('jsonwebtoken');
const JWT_SECRET = process.env.JWT_SECRET;

// Middleware to verify JWT token
const authenticateToken = (req, res, next) => {
    // Get token from the Authorization header ('Bearer TOKEN')
    const authHeader = req.headers['authorization'];
    const token = authHeader && authHeader.split(' ')[1];

    if (token == null) {
        // No token provided
        return res.status(401).json({ message: 'Zugriff verweigert. Kein Token bereitgestellt.' }); // Unauthorized
    }

    // Verify the token
    jwt.verify(token, JWT_SECRET, (err, user) => {
        if (err) {
            console.error("JWT Verification Error:", err.message);
            if (err.name === 'TokenExpiredError') {
                return res.status(401).json({ message: 'Token abgelaufen.' });
            }
            return res.status(403).json({ message: 'Zugriff verweigert. Ungültiges Token.' }); // Forbidden
        }

        // Token is valid, attach user payload (with id and role) to the request object
        req.user = user; // Contains { userId, username, role } from the JWT payload
        next(); // Proceed to the next middleware or route handler
    });
};

// Middleware to check for specific roles (e.g., 'admin')
// Usage: router.post('/', authenticateToken, authorizeRole('admin'), (req, res) => { ... });
const authorizeRole = (requiredRole) => {
    return (req, res, next) => {
        if (!req.user || req.user.role !== requiredRole) {
            return res.status(403).json({ message: `Zugriff verweigert. Rolle '${requiredRole}' erforderlich.` }); // Forbidden
        }
        next(); // User has the required role
    };
};

// Middleware to check for multiple roles (e.g., 'admin' or 'referee')
const authorizeRoles = (...allowedRoles) => {
    return (req, res, next) => {
        if (!req.user || !allowedRoles.includes(req.user.role)) {
            return res.status(403).json({ message: `Zugriff verweigert. Erforderliche Rollen: ${allowedRoles.join(' oder ')}.` }); // Forbidden
        }
        next(); // User has one of the allowed roles
    };
};


module.exports = {
    authenticateToken,
    authorizeRole,
    authorizeRoles
};
