// src/routes/tournamentRoutes.js
const express = require('express');
const tournamentController = require('../controllers/tournamentController');
// Import authentication middleware
const { authenticateToken, authorizeRole } = require('../middleware/authMiddleware');

const router = express.Router();

// Middleware to validate ID format
const validateIdFormat = (req, res, next) => {
  const idParam = req.params.id;
  if (!/^[0-9a-fA-F\-]+$/.test(idParam)) {
    return res.status(400).json({ message: 'Invalid ID format' });
  }
  next();
};

// --- Publicly Accessible GET Routes ---
// GET /api/tournaments - Get all tournaments
router.get('/', tournamentController.getAllTournaments);

// GET /api/tournaments/:id - Get a specific tournament
// Needs to be before routes with more specific parameters like /export or /:id/players
router.get('/:id', validateIdFormat, tournamentController.getTournamentById);

// GET /api/tournaments/:id/players - Get players for a specific tournament
router.get('/:tournamentId/players', tournamentController.getTournamentPlayers);

// --- Admin only routes (require authentication and 'admin' role) ---
// POST /api/tournaments - Create a new tournament
router.post('/', authenticateToken, authorizeRole('admin'), tournamentController.createTournament);

// PUT /api/tournaments/:id - Update a tournament
router.put('/:id', validateIdFormat, authenticateToken, authorizeRole('admin'), tournamentController.updateTournament);

// DELETE /api/tournaments/:id - Delete a tournament
router.delete('/:id', validateIdFormat, authenticateToken, authorizeRole('admin'), tournamentController.deleteTournament);

// --- NEW: Routes for managing tournament players (Admin only) ---

// POST /api/tournaments/:tournamentId/players - Add an existing player to a tournament
router.post('/:tournamentId/players', authenticateToken, authorizeRole('admin'), tournamentController.addPlayerToTournament);

// DELETE /api/tournaments/:tournamentId/players/:playerId - Remove a player from a tournament
router.delete('/:tournamentId/players/:playerId', authenticateToken, authorizeRole('admin'), tournamentController.removePlayerFromTournament);

// --- Placeholder routes for future implementation (Admin only) ---

// POST /api/tournaments/import - Import tournaments from CSV
router.post('/import', authenticateToken, authorizeRole('admin'), tournamentController.importTournaments);

// GET /api/tournaments/export - Export tournaments to CSV
// Placed after specific ID routes to avoid conflict
router.get('/export', authenticateToken, authorizeRole('admin'), tournamentController.exportTournaments);

// POST /api/tournaments/:id/logo - Add/Update logo for a tournament
router.post('/:id/logo', validateIdFormat, authenticateToken, authorizeRole('admin'), tournamentController.addLogo);

module.exports = router;