// src/utils/generateCertificate.js
const child_process = require('child_process');
const fs = require('fs');
const path = require('path');

const keyPath = path.join(__dirname, '..', '..', 'key.pem'); // Place in root directory
const certPath = path.join(__dirname, '..', '..', 'cert.pem'); // Place in root directory

// Check if files already exist
if (fs.existsSync(keyPath) && fs.existsSync(certPath)) {
    console.log('Certificate files (key.pem, cert.pem) already exist. Skipping generation.');
    console.log('Delete the files if you want to regenerate them.');
    process.exit(0); // Exit successfully
}

// OpenSSL command to generate a self-signed certificate
// -x509: Output a self-signed certificate instead of a certificate request.
// -nodes: Don't encrypt the private key.
// -days 365: Validity period.
// -newkey rsa:2048: Generate a new RSA key of 2048 bits.
// -keyout: Path to write the private key.
// -out: Path to write the certificate.
// -subj: Subject information (avoids interactive prompts). Customize as needed.
const opensslCommand = `openssl req -x509 -nodes -days 365 -newkey rsa:2048 -keyout "${keyPath}" -out "${certPath}" -subj "/C=DE/ST=Niedersachsen/L=Hannover/O=TT Tournament Dev/OU=Development/CN=localhost"`;

console.log('Generating self-signed certificate...');
console.log(`Executing: ${opensslCommand}`);

try {
    // Execute the command synchronously
    child_process.execSync(opensslCommand, { stdio: 'inherit' }); // Show output/errors
    console.log('Successfully generated key.pem and cert.pem');
    console.log(`Key file: ${keyPath}`);
    console.log(`Certificate file: ${certPath}`);
    console.log('\nNote: Your browser will likely show a security warning for this self-signed certificate.');
} catch (error) {
    console.error('\n-----------------------------------------');
    console.error('Error generating certificate:', error.message);
    console.error('-----------------------------------------');
    console.error('Please ensure OpenSSL is installed and accessible in your system PATH.');
    console.error('Command failed:', opensslCommand);
    process.exit(1); // Exit with error code
}
